;;; md-data.scm --- Paradoxically schemey machine-dependent data

;;	Copyright (C) 2003,2004 Free Software Foundation, Inc.
;;
;; This program is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.
;;
;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this software; see the file COPYING.  If not, write to
;; the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
;; Boston, MA 02111-1307 USA
;;
;; As a special exception, the Free Software Foundation gives permission
;; for additional uses of the text contained in its release of GUILE.
;;
;; The exception is that, if you link the GUILE library with other files
;; to produce an executable, this does not by itself cause the
;; resulting executable to be covered by the GNU General Public License.
;; Your use of that executable is in no way restricted on account of
;; linking the GUILE library code into it.
;;
;; This exception does not however invalidate any other reasons why
;; the executable file might be covered by the GNU General Public License.
;;
;; This exception applies only to the code released by the
;; Free Software Foundation under the name GUILE.  If you copy
;; code from other Free Software Foundation releases into a copy of
;; GUILE, as the General Public License permits, the exception does
;; not apply to the code that you add in this way.  To avoid misleading
;; anyone as to the status of such modified files, you must delete
;; this exception notice from them.
;;
;; If you write modifications of your own for GUILE, it is your choice
;; whether to permit this exception to apply to your modifications.
;; If you do not wish that, delete this exception notice.

;;; Commentary:

;; This module exports the following procs:
;;   (all-keywords) => list of KEYWORD/DESCRIPTION pairs
;;   (all-md-names) => NAME list
;;   (md-by-name NAME) => BODY-DATA
;;   (md-body-lookup BODY-DATA KEYWORD) => OBJ
;;   (extend-md-table! NAME BODY-DATA)
;;
;; NAME is a symbol, BODY-DATA is a list of alternating keyword and
;; associated-data elements.  DESCRIPTION is a string.

;;; Code:

(define-module (ice-9 md-data)
  #:export (all-keywords
            all-md-names
            md-by-name
            md-body-lookup
            extend-md-table!))

(define *keywords*
  '((#:magic . "string to be found at the beginning of a file")))

(define (all-keywords)
  (list-copy *keywords*))

(define *md*
  `((ELF
     #:magic ,(string #\177 #\E #\L #\F))
    (Mach-O-BE
     #:magic ,(list->string (map integer->char '(#xfe #xed #xfa #xce))))))

(define (all-md-names)
  (map car *md*))

(define (md-by-name name)
  (assq-ref *md* name))

(define (md-body-lookup body kw)
  (let loop ((ls body))
    (if (null? ls)
        #f
        (if (eq? kw (car ls))
            (cadr ls)
            (loop (cddr ls))))))

(define (extend-md-table! name . info)
  (cond ((assq name *md*)
         => (lambda (cell)
              (set-cdr! cell info)))
        (else
         (set! *md* (acons name info *md*)))))

;;; md-data.scm ends here
